@@ -54,7 +54,6 @@
    ekr@rtfm.com  Sun Jan 17 17:57:15 1999
  */
 
-static char *RCSSTRING="$Id: r_assoc.c,v 1.4 2001/12/24 06:06:26 ekr Exp $";
 
 #include <r_common.h>
 #include "r_assoc.h"
@@ -122,6 +121,9 @@ int r_assoc_destroy(assocp)
     for(i=0;i<assoc->size;i++)
       destroy_assoc_chain(assoc->chains[i]);
 
+    free(assoc->chains);
+    free(assoc);
+
     return(0);
   }
 
@@ -174,7 +176,7 @@ static int copy_assoc_chain(newp,old)
       ptr->copy=old->copy;
 	
       if(old->copy){
-	if(r=old->copy(&ptr->data,old->data))
+	if((r=old->copy(&ptr->data,old->data)))
 	  ABORT(r);
       }
       else
@@ -225,7 +227,7 @@ int r_assoc_fetch(assoc,key,len,datap)
     r_assoc_el *bucket;
     int r;
 
-    if(r=r_assoc_fetch_bucket(assoc,key,len,&bucket)){
+    if((r=r_assoc_fetch_bucket(assoc,key,len,&bucket))){
       if(r!=R_NOT_FOUND)
 	ERETURN(r);
       return(r);
@@ -247,7 +249,7 @@ int r_assoc_insert(assoc,key,len,data,copy,destroy,how)
     r_assoc_el *bucket,*new_bucket=0;
     int r,_status;
     
-    if(r=r_assoc_fetch_bucket(assoc,key,len,&bucket)){
+    if((r=r_assoc_fetch_bucket(assoc,key,len,&bucket))){
       /*Note that we compute the hash value twice*/
       UINT4 hash_value;
 
@@ -305,7 +307,7 @@ int r_assoc_copy(newp,old)
     if(!(new->chains=(r_assoc_el **)calloc(sizeof(r_assoc_el),old->size)))
       ABORT(R_NO_MEMORY);
     for(i=0;i<new->size;i++){
-      if(r=copy_assoc_chain(new->chains+i,old->chains[i]))
+      if((r=copy_assoc_chain(new->chains+i,old->chains[i])))
 	ABORT(R_NO_MEMORY);
     }
     *newp=new;
